package
{
	import flash.display.Graphics;
	import flash.geom.Point;
	
	public class Bezier2DEx implements IEdge
	{
        private var baseLine:Bezier2D;
        private var segment:uint = 100; //分割数
        private var poses1:Array; //tに対応した位置
        private var poses2:Array;
        private var thicknesses:Object;
        private var color:uint = 0;
        private var alpha:Number = 1;
        public var _start_t:Number;
        public var _end_t:Number;

        public function get start_t():Number{ return _start_t; }
        public function set start_t(t:Number):void{ _start_t = t; }
        public function get end_t():Number{ return _end_t; }
        public function set end_t(t:Number):void{ _end_t = t; }
        
        public function get length():Number{
            return baseLine.length;
        }
        
		public function Bezier2DEx(_p0:Point , _p1:Point , _p2:Point)
		{
            baseLine = new Bezier2D(_p0 , _p1 , _p2);
            _start_t = 0;
            _end_t = 1;
            setThickness(1, 1);
		}

        private function getTangentPoint(isIn:Boolean, distance:Number, t:Number, isControl:Boolean = false):Point
        {
            var pt:Point = new Point();
            var diff:Point = baseLine.diff(t);
            var angle:Number = -Math.atan2(diff.x, diff.y);
            var disX:Number = Math.cos(angle) * distance;
            var disY:Number = Math.sin(angle) * distance;
            var basePt:Point;
            
            if(isControl == false)
            {
            	basePt = baseLine.f(t);
	            if(isIn)
	            {
	                pt.x = basePt.x + disX;
	                pt.y = basePt.y + disY;
	            }
	            else
	            {
	                pt.x = basePt.x - disX;
	                pt.y = basePt.y - disY;
	            }
            }
            else
            {
            	basePt = new Point(baseLine.control.x, baseLine.control.y);
            	var offset:Number = isIn ? 0.5 : - 0.5;
            	pt.x = basePt.x + disX * offset;
                pt.y = basePt.y + disY * offset;
            }

            return pt;
        }
        
        private function setPoses():void
        {
            poses1 = new Array(segment);
            poses2 = new Array(segment);
            
            var startPt1:Point = getTangentPoint(true, thicknesses.start / 2, 0);
            var ctrlPt1:Point = getTangentPoint(true, (thicknesses.start + thicknesses.end) / 2, 0.5, true);
            var endPt1:Point = getTangentPoint(true, thicknesses.end / 2, 1);
            var startPt2:Point = getTangentPoint(false, thicknesses.start / 2, 0);
            var ctrlPt2:Point = getTangentPoint(false, (thicknesses.start + thicknesses.end) / 2, 0.5, true);
            var endPt2:Point = getTangentPoint(false, thicknesses.end / 2, 1);
            var bezier1:Bezier2D = new Bezier2D(startPt1, endPt1, ctrlPt1);
            var bezier2:Bezier2D = new Bezier2D(startPt2, endPt2, ctrlPt2);
            var i:uint;
            var t:Number = 0;
            var dt:Number = 1 / segment;

            for(i = 0; i < segment; i++)
            {
               poses1[i] = bezier1.f(t);
               poses2[i] = bezier2.f(t);
               t += dt;
            }
        }
        
        public function setThickness(start:Number, end:Number):void
        {
            if(isNaN(end)){end = start}
            thicknesses = 
            {
                start:start, end:end
            }
            setPoses();
        }
        
        private function calcToSegment(target_t:Number):uint
        {
            var seg:uint = (segment -1) * target_t;
            return seg;
        }
        
        public function draw(g:Graphics):void
        {
            var startSeg:uint = calcToSegment(_start_t);
            var endSeg:uint = calcToSegment(_end_t);
            var cnt:int = startSeg;
            g.moveTo(poses1[startSeg].x, poses1[startSeg].y);
            while(cnt <= endSeg)
            {
                g.lineTo(poses1[cnt].x, poses1[cnt].y);
                cnt++;
            }
            g.lineTo(poses2[endSeg].x, poses2[endSeg].y);
            
            cnt = endSeg;
            while(cnt >= startSeg)
            {
                g.lineTo(poses2[cnt].x, poses2[cnt].y);
                cnt--;
            }
            g.lineTo(poses1[startSeg].x, poses1[startSeg].y);
        }
        
        public function getT_byLength(length:Number):Number
        {
        	return baseLine.length2T(length, 0.1);
        }
        
        public function set_t(start_t:Number, end_t:Number):void
        {
            this.start_t = start_t;
            this.end_t = end_t;
        }      
	}
}